@@ -51,7 +51,7 @@ impl<'a> GrowableBoolean<'a> {
         let validity = std::mem::take(&mut self.validity);
         let values = std::mem::take(&mut self.values);
 
-        BooleanArray::from_data(self.data_type.clone(), values.into(), validity.into())
+        BooleanArray::new(self.data_type.clone(), values.into(), validity.into())
     }
 }
 
@@ -82,6 +82,6 @@ impl<'a> Growable<'a> for GrowableBoolean<'a> {
 
 impl<'a> From<GrowableBoolean<'a>> for BooleanArray {
     fn from(val: GrowableBoolean<'a>) -> Self {
-        BooleanArray::from_data(val.data_type, val.values.into(), val.validity.into())
+        BooleanArray::new(val.data_type, val.values.into(), val.validity.into())
     }
 }
