@@ -278,6 +278,9 @@ impl<T: NativeType> Array for PrimitiveArray<T> {
     fn with_validity(&self, validity: Option<Bitmap>) -> Box<dyn Array> {
         Box::new(self.with_validity(validity))
     }
+    fn to_boxed(&self) -> Box<dyn Array> {
+        Box::new(self.clone())
+    }
 }
 
 /// A type definition [`PrimitiveArray`] for `i8`
