@@ -3,7 +3,7 @@ use std::ops::Add;
 use multiversion::multiversion;
 
 use crate::bitmap::utils::{BitChunkIterExact, BitChunksExact};
-use crate::datatypes::{DataType, IntervalUnit};
+use crate::datatypes::{DataType, PhysicalType, PrimitiveType};
 use crate::error::{ArrowError, Result};
 use crate::scalar::*;
 use crate::types::simd::*;
@@ -104,68 +104,54 @@ where
     }
 }
 
-macro_rules! dyn_sum {
-    ($ty:ty, $array:expr) => {{
-        let array = $array
-            .as_any()
-            .downcast_ref::<PrimitiveArray<$ty>>()
-            .unwrap();
-        Box::new(PrimitiveScalar::<$ty>::new(
-            $array.data_type().clone(),
-            sum_primitive::<$ty>(array),
-        ))
-    }};
+/// Whether [`sum`] supports `data_type`
+pub fn can_sum(data_type: &DataType) -> bool {
+    if let PhysicalType::Primitive(primitive) = data_type.to_physical_type() {
+        use PrimitiveType::*;
+        matches!(
+            primitive,
+            Int8 | Int16 | Int64 | Int128 | UInt8 | UInt16 | UInt32 | UInt64 | Float32 | Float64
+        )
+    } else {
+        false
+    }
 }
 
-/// Whether [`sum`] is valid for `data_type`
-pub fn can_sum(data_type: &DataType) -> bool {
-    use DataType::*;
-    matches!(
-        data_type,
-        Int8 | Int16
-            | Date32
-            | Time32(_)
-            | Interval(IntervalUnit::YearMonth)
-            | Int64
-            | Date64
-            | Time64(_)
-            | Timestamp(_, _)
-            | Duration(_)
-            | UInt8
-            | UInt16
-            | UInt32
-            | UInt64
-            | Float32
-            | Float64
-    )
-}
+macro_rules! with_match_primitive_type {(
+    $key_type:expr, | $_:tt $T:ident | $($body:tt)*
+) => ({
+    macro_rules! __with_ty__ {( $_ $T:ident ) => ( $($body)* )}
+    use crate::datatypes::PrimitiveType::*;
+    match $key_type {
+        Int8 => __with_ty__! { i8 },
+        Int16 => __with_ty__! { i16 },
+        Int32 => __with_ty__! { i32 },
+        Int64 => __with_ty__! { i64 },
+        Int128 => __with_ty__! { i128 },
+        UInt8 => __with_ty__! { u8 },
+        UInt16 => __with_ty__! { u16 },
+        UInt32 => __with_ty__! { u32 },
+        UInt64 => __with_ty__! { u64 },
+        Float32 => __with_ty__! { f32 },
+        Float64 => __with_ty__! { f64 },
+        _ => return Err(ArrowError::InvalidArgumentError(format!(
+            "`sum` operator do not support primitive `{:?}`",
+            $key_type,
+        ))),
+    }
+})}
 
 /// Returns the sum of all elements in `array` as a [`Scalar`] of the same physical
 /// and logical types as `array`.
 /// # Error
 /// Errors iff the operation is not supported.
 pub fn sum(array: &dyn Array) -> Result<Box<dyn Scalar>> {
-    Ok(match array.data_type() {
-        DataType::Int8 => dyn_sum!(i8, array),
-        DataType::Int16 => dyn_sum!(i16, array),
-        DataType::Int32
-        | DataType::Date32
-        | DataType::Time32(_)
-        | DataType::Interval(IntervalUnit::YearMonth) => {
-            dyn_sum!(i32, array)
-        }
-        DataType::Int64
-        | DataType::Date64
-        | DataType::Time64(_)
-        | DataType::Timestamp(_, _)
-        | DataType::Duration(_) => dyn_sum!(i64, array),
-        DataType::UInt8 => dyn_sum!(u8, array),
-        DataType::UInt16 => dyn_sum!(u16, array),
-        DataType::UInt32 => dyn_sum!(u32, array),
-        DataType::UInt64 => dyn_sum!(u64, array),
-        DataType::Float16 => unreachable!(),
-        DataType::Float32 => dyn_sum!(f32, array),
-        DataType::Float64 => dyn_sum!(f64, array),
+    Ok(match array.data_type().to_physical_type() {
+        PhysicalType::Primitive(primitive) => with_match_primitive_type!(primitive, |$T| {
+            let data_type = array.data_type().clone();
+            let array = array.as_any().downcast_ref().unwrap();
+            Box::new(PrimitiveScalar::new(data_type, sum_primitive::<$T>(array)))
+        }),
         _ => {
             return Err(ArrowError::InvalidArgumentError(format!(
                 "The `sum` operator does not support type `{:?}`",
