@@ -1,8 +1,10 @@
 use std::hash::Hash;
 
-use num_traits::{AsPrimitive, Float};
+use num_traits::{AsPrimitive, Float, ToPrimitive};
 
+use crate::datatypes::IntervalUnit;
 use crate::error::Result;
+use crate::types::{days_ms, months_days_ns};
 use crate::{
     array::*,
     bitmap::Bitmap,
@@ -66,7 +68,7 @@ pub fn primitive_to_boolean<T: NativeType>(
     let iter = from.values().iter().map(|v| *v != T::default());
     let values = Bitmap::from_trusted_len_iter(iter);
 
-    BooleanArray::from_data(to_type, values, from.validity().cloned())
+    BooleanArray::new(to_type, values, from.validity().cloned())
 }
 
 pub(super) fn primitive_to_boolean_dyn<T>(
@@ -203,10 +205,8 @@ pub fn float_to_decimal<T>(
     to_scale: usize,
 ) -> PrimitiveArray<i128>
 where
-    T: NativeType + Float,
+    T: NativeType + Float + ToPrimitive,
     f64: AsPrimitive<T>,
-    i128: From<T>,
-    T: AsPrimitive<f64>,
 {
     // 1.2 => 12
     let multiplier: T = (10_f64).powi(to_scale as i32).as_();
@@ -218,7 +218,7 @@ where
 
     let values = from.iter().map(|x| {
         x.and_then(|x| {
-            let x = i128::from(*x * multiplier);
+            let x = (*x * multiplier).to_i128().unwrap();
             if x > max_for_precision || x < min_for_precision {
                 None
             } else {
@@ -237,10 +237,11 @@ pub(super) fn float_to_decimal_dyn<T>(
     scale: usize,
 ) -> Result<Box<dyn Array>>
 where
-    T: NativeType + AsPrimitive<i128>,
+    T: NativeType + Float + ToPrimitive,
+    f64: AsPrimitive<T>,
 {
     let from = from.as_any().downcast_ref().unwrap();
-    Ok(Box::new(integer_to_decimal::<T>(from, precision, scale)))
+    Ok(Box::new(float_to_decimal::<T>(from, precision, scale)))
 }
 
 /// Cast [`PrimitiveArray`] as a [`PrimitiveArray`]
@@ -265,7 +266,7 @@ pub fn primitive_to_same_primitive<T>(
 where
     T: NativeType,
 {
-    PrimitiveArray::<T>::from_data(
+    PrimitiveArray::<T>::new(
         to_type.clone(),
         from.values().clone(),
         from.validity().cloned(),
@@ -552,3 +553,31 @@ pub fn naive_timestamp_to_utf8<O: Offset>(
         }
     }
 }
+
+#[inline]
+fn days_ms_to_months_days_ns_scalar(from: days_ms) -> months_days_ns {
+    months_days_ns::new(0, from.days(), from.milliseconds() as i64 * 1000)
+}
+
+/// Casts [`days_ms`]s to [`months_days_ns`]. This operation is infalible and lossless.
+pub fn days_ms_to_months_days_ns(from: &PrimitiveArray<days_ms>) -> PrimitiveArray<months_days_ns> {
+    unary(
+        from,
+        days_ms_to_months_days_ns_scalar,
+        DataType::Interval(IntervalUnit::MonthDayNano),
+    )
+}
+
+#[inline]
+fn months_to_months_days_ns_scalar(from: i32) -> months_days_ns {
+    months_days_ns::new(from, 0, 0)
+}
+
+/// Casts months represented as [`i32`]s to [`months_days_ns`]. This operation is infalible and lossless.
+pub fn months_to_months_days_ns(from: &PrimitiveArray<i32>) -> PrimitiveArray<months_days_ns> {
+    unary(
+        from,
+        months_to_months_days_ns_scalar,
+        DataType::Interval(IntervalUnit::MonthDayNano),
+    )
+}
