@@ -17,6 +17,10 @@ fn is_like_pattern(c: char) -> bool {
     c == '%' || c == '_'
 }
 
+fn replace_pattern(pattern: &str) -> String {
+    pattern.replace('%', ".*").replace('_', ".")
+}
+
 #[inline]
 fn a_like_utf8<O: Offset, F: Fn(bool) -> bool>(
     lhs: &Utf8Array<O>,
@@ -40,7 +44,7 @@ fn a_like_utf8<O: Offset, F: Fn(bool) -> bool>(
                     let pattern = if let Some(pattern) = map.get(pattern) {
                         pattern
                     } else {
-                        let re_pattern = pattern.replace("%", ".*").replace("_", ".");
+                        let re_pattern = replace_pattern(pattern);
                         let re = Regex::new(&format!("^{}$", re_pattern)).map_err(|e| {
                             ArrowError::InvalidArgumentError(format!(
                                 "Unable to build regex from LIKE pattern: {}",
@@ -56,7 +60,7 @@ fn a_like_utf8<O: Offset, F: Fn(bool) -> bool>(
             }
         }))?;
 
-    Ok(BooleanArray::from_data(DataType::Boolean, values, validity))
+    Ok(BooleanArray::new(DataType::Boolean, values, validity))
 }
 
 /// Returns `lhs LIKE rhs` operation on two [`Utf8Array`].
@@ -103,7 +107,7 @@ fn a_like_utf8_scalar<O: Offset, F: Fn(bool) -> bool>(
     let validity = lhs.validity();
 
     let values = if !rhs.contains(is_like_pattern) {
-        Bitmap::from_trusted_len_iter(lhs.values_iter().map(|x| x == rhs))
+        Bitmap::from_trusted_len_iter(lhs.values_iter().map(|x| op(x == rhs)))
     } else if rhs.ends_with('%') && !rhs[..rhs.len() - 1].contains(is_like_pattern) {
         // fast path, can use starts_with
         let starts_with = &rhs[..rhs.len() - 1];
@@ -113,7 +117,7 @@ fn a_like_utf8_scalar<O: Offset, F: Fn(bool) -> bool>(
         let ends_with = &rhs[1..];
         Bitmap::from_trusted_len_iter(lhs.values_iter().map(|x| op(x.ends_with(ends_with))))
     } else {
-        let re_pattern = rhs.replace("%", ".*").replace("_", ".");
+        let re_pattern = replace_pattern(rhs);
         let re = Regex::new(&format!("^{}$", re_pattern)).map_err(|e| {
             ArrowError::InvalidArgumentError(format!(
                 "Unable to build regex from LIKE pattern: {}",
@@ -122,7 +126,7 @@ fn a_like_utf8_scalar<O: Offset, F: Fn(bool) -> bool>(
         })?;
         Bitmap::from_trusted_len_iter(lhs.values_iter().map(|x| op(re.is_match(x))))
     };
-    Ok(BooleanArray::from_data(
+    Ok(BooleanArray::new(
         DataType::Boolean,
         values,
         validity.cloned(),
@@ -187,10 +191,8 @@ fn a_like_binary<O: Offset, F: Fn(bool) -> bool>(
                     let pattern = if let Some(pattern) = map.get(pattern) {
                         pattern
                     } else {
-                        let re_pattern = simdutf8::basic::from_utf8(pattern)
-                            .unwrap()
-                            .replace("%", ".*")
-                            .replace("_", ".");
+                        let re_pattern = simdutf8::basic::from_utf8(pattern).unwrap();
+                        let re_pattern = replace_pattern(re_pattern);
                         let re = BytesRegex::new(&format!("^{}$", re_pattern)).map_err(|e| {
                             ArrowError::InvalidArgumentError(format!(
                                 "Unable to build regex from LIKE pattern: {}",
@@ -206,7 +208,7 @@ fn a_like_binary<O: Offset, F: Fn(bool) -> bool>(
             }
         }))?;
 
-    Ok(BooleanArray::from_data(DataType::Boolean, values, validity))
+    Ok(BooleanArray::new(DataType::Boolean, values, validity))
 }
 
 /// Returns `lhs LIKE rhs` operation on two [`BinaryArray`].
@@ -260,7 +262,7 @@ fn a_like_binary_scalar<O: Offset, F: Fn(bool) -> bool>(
     })?;
 
     let values = if !pattern.contains(is_like_pattern) {
-        Bitmap::from_trusted_len_iter(lhs.values_iter().map(|x| x == rhs))
+        Bitmap::from_trusted_len_iter(lhs.values_iter().map(|x| op(x == rhs)))
     } else if pattern.ends_with('%') && !pattern[..pattern.len() - 1].contains(is_like_pattern) {
         // fast path, can use starts_with
         let starts_with = &rhs[..rhs.len() - 1];
@@ -270,7 +272,7 @@ fn a_like_binary_scalar<O: Offset, F: Fn(bool) -> bool>(
         let ends_with = &rhs[1..];
         Bitmap::from_trusted_len_iter(lhs.values_iter().map(|x| op(x.ends_with(ends_with))))
     } else {
-        let re_pattern = pattern.replace("%", ".*").replace("_", ".");
+        let re_pattern = replace_pattern(pattern);
         let re = BytesRegex::new(&format!("^{}$", re_pattern)).map_err(|e| {
             ArrowError::InvalidArgumentError(format!(
                 "Unable to build regex from LIKE pattern: {}",
@@ -279,7 +281,7 @@ fn a_like_binary_scalar<O: Offset, F: Fn(bool) -> bool>(
         })?;
         Bitmap::from_trusted_len_iter(lhs.values_iter().map(|x| op(re.is_match(x))))
     };
-    Ok(BooleanArray::from_data(
+    Ok(BooleanArray::new(
         DataType::Boolean,
         values,
         validity.cloned(),
