@@ -7,6 +7,7 @@ pub use csv::{ByteRecord, Reader, ReaderBuilder};
 
 mod infer_schema;
 
+pub use super::utils::infer;
 pub use deserialize::{deserialize_batch, deserialize_column};
-pub use infer_schema::{infer, infer_schema};
+pub use infer_schema::infer_schema;
 pub use reader::*;
