@@ -2,25 +2,9 @@ use std::io::Read;
 
 use super::{ByteRecord, Reader};
 
-use crate::{
-    datatypes::*,
-    error::{ArrowError, Result},
-};
+use crate::error::{ArrowError, Result};
 
-/// Returns a new [`Schema`] whereby the fields are selected based on `projection`.
-pub fn projected_schema(schema: &Schema, projection: Option<&[usize]>) -> Schema {
-    match &projection {
-        Some(projection) => {
-            let fields = schema.fields();
-            let projected_fields: Vec<Field> =
-                projection.iter().map(|i| fields[*i].clone()).collect();
-            Schema::new_from(projected_fields, schema.metadata().clone())
-        }
-        None => schema.clone(),
-    }
-}
-
-/// Reads `len` rows from `reader` into `row`, skiping `skip`.
+/// Reads `len` rows from `reader` into `row`, skiping the first `skip`.
 /// This operation has minimal CPU work and is thus the fastest way to read through a CSV
 /// without deserializing the contents to Arrow.
 pub fn read_rows<R: Read>(
