@@ -14,6 +14,7 @@ use super::super::super::iterator::{BufStreamingIterator, StreamingIterator};
 use crate::array::{DictionaryArray, DictionaryKey, Offset};
 use csv_core::WriteResult;
 use std::any::Any;
+use std::fmt::{Debug, Write};
 
 /// Options to serialize logical types to CSV
 /// The default is to format times and dates as `chrono` crate formats them.
@@ -49,6 +50,16 @@ impl Default for SerializeOptions {
     }
 }
 
+/// Utility to write to `&mut Vec<u8>` buffer
+struct StringWrap<'a>(pub &'a mut Vec<u8>);
+
+impl<'a> Write for StringWrap<'a> {
+    fn write_str(&mut self, s: &str) -> std::fmt::Result {
+        self.0.extend_from_slice(s.as_bytes());
+        Ok(())
+    }
+}
+
 fn primitive_write<'a, T: NativeType + ToLexical>(
     array: &'a PrimitiveArray<T>,
 ) -> Box<dyn StreamingIterator<Item = [u8]> + 'a> {
@@ -81,7 +92,8 @@ macro_rules! dyn_date {
                 array.iter(),
                 move |x, buf| {
                     if let Some(x) = x {
-                        buf.extend_from_slice(($fn)(*x).format(format).to_string().as_bytes())
+                        let dt = ($fn)(*x).format(format);
+                        let _ = write!(StringWrap(buf), "{}", dt);
                     }
                 },
                 vec![],
@@ -91,7 +103,8 @@ macro_rules! dyn_date {
                 array.iter(),
                 move |x, buf| {
                     if let Some(x) = x {
-                        buf.extend_from_slice(($fn)(*x).to_string().as_bytes())
+                        let dt = ($fn)(*x);
+                        let _ = write!(StringWrap(buf), "{}", dt);
                     }
                 },
                 vec![],
@@ -111,10 +124,8 @@ fn timestamp_with_tz_default<'a>(
             array.iter(),
             move |x, buf| {
                 if let Some(x) = x {
-                    let data =
-                        temporal_conversions::timestamp_to_datetime(*x, time_unit, &timezone)
-                            .to_string();
-                    buf.extend_from_slice(data.as_bytes())
+                    let dt = temporal_conversions::timestamp_to_datetime(*x, time_unit, &timezone);
+                    let _ = write!(StringWrap(buf), "{}", dt);
                 }
             },
             vec![],
@@ -126,10 +137,9 @@ fn timestamp_with_tz_default<'a>(
                 array.iter(),
                 move |x, buf| {
                     if let Some(x) = x {
-                        let data =
-                            temporal_conversions::timestamp_to_datetime(*x, time_unit, &timezone)
-                                .to_string();
-                        buf.extend_from_slice(data.as_bytes())
+                        let dt =
+                            temporal_conversions::timestamp_to_datetime(*x, time_unit, &timezone);
+                        let _ = write!(StringWrap(buf), "{}", dt);
                     }
                 },
                 vec![],
@@ -157,11 +167,9 @@ fn timestamp_with_tz_with_format<'a>(
             array.iter(),
             move |x, buf| {
                 if let Some(x) = x {
-                    let data =
-                        temporal_conversions::timestamp_to_datetime(*x, time_unit, &timezone)
-                            .format(format)
-                            .to_string();
-                    buf.extend_from_slice(data.as_bytes())
+                    let dt = temporal_conversions::timestamp_to_datetime(*x, time_unit, &timezone)
+                        .format(format);
+                    let _ = write!(StringWrap(buf), "{}", dt);
                 }
             },
             vec![],
@@ -173,11 +181,10 @@ fn timestamp_with_tz_with_format<'a>(
                 array.iter(),
                 move |x, buf| {
                     if let Some(x) = x {
-                        let data =
+                        let dt =
                             temporal_conversions::timestamp_to_datetime(*x, time_unit, &timezone)
-                                .format(format)
-                                .to_string();
-                        buf.extend_from_slice(data.as_bytes())
+                                .format(format);
+                        let _ = write!(StringWrap(buf), "{}", dt);
                     }
                 },
                 vec![],
@@ -216,8 +223,7 @@ fn new_utf8_serializer<'a, O: Offset>(
         .delimiter(options.delimiter)
         .build();
 
-    let resize = |local_buf: &mut Vec<u8>| {
-        let additional = local_buf.len();
+    let resize = |local_buf: &mut Vec<u8>, additional: usize| {
         local_buf.extend(std::iter::repeat(0u8).take(additional))
     };
 
@@ -229,16 +235,15 @@ fn new_utf8_serializer<'a, O: Offset>(
                 // This will ensure a csv parser will not read them as missing
                 // in a delimited field
                 Some("") => buf.extend_from_slice(b"\"\""),
-                Some(s) => loop {
-                    // first write field
+                Some(s) => {
+                    if s.len() < local_buf.len() * 3 {
+                        resize(&mut local_buf, s.len() * 3)
+                    }
                     match ser_writer.field(s.as_bytes(), &mut local_buf) {
-                        (WriteResult::OutputFull, _, _) => resize(&mut local_buf),
-                        // then on success write delimiter
-                        // we need to make this call because we might need to end with quotes
                         (WriteResult::InputEmpty, _, n_out) => {
                             // the writer::delimiter call writes a maximum of 2 bytes
                             if local_buf.len() - n_out < 2 {
-                                resize(&mut local_buf);
+                                resize(&mut local_buf, 2);
                             }
                             match ser_writer.delimiter(&mut local_buf[n_out..]) {
                                 (WriteResult::InputEmpty, n_out_delimiter) => {
@@ -249,10 +254,10 @@ fn new_utf8_serializer<'a, O: Offset>(
                                 }
                                 _ => unreachable!(),
                             }
-                            break;
                         }
+                        _ => unreachable!(),
                     }
-                },
+                }
                 _ => {}
             }
         },
