@@ -1,34 +1,20 @@
-// Licensed to the Apache Software Foundation (ASF) under one
-// or more contributor license agreements.  See the NOTICE file
-// distributed with this work for additional information
-// regarding copyright ownership.  The ASF licenses this file
-// to you under the Apache License, Version 2.0 (the
-// "License"); you may not use this file except in compliance
-// with the License.  You may obtain a copy of the License at
-//
-//   http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing,
-// software distributed under the License is distributed on an
-// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-// KIND, either express or implied.  See the License for the
-// specific language governing permissions and limitations
-// under the License.
-
 //! Arrow IPC File and Stream Writers
 //!
 //! The `FileWriter` and `StreamWriter` have similar interfaces,
 //! however the `FileWriter` expects a reader that supports `Seek`ing
 
 use std::io::Write;
+use std::sync::Arc;
 
-use super::common::{encoded_batch, DictionaryTracker, EncodedData, WriteOptions};
+use super::super::IpcField;
+use super::common::{encode_chunk, DictionaryTracker, EncodedData, WriteOptions};
 use super::common_sync::{write_continuation, write_message};
-use super::schema_to_bytes;
+use super::{default_ipc_fields, schema_to_bytes};
 
+use crate::array::Array;
+use crate::chunk::Chunk;
 use crate::datatypes::*;
 use crate::error::{ArrowError, Result};
-use crate::record_batch::RecordBatch;
 
 /// Arrow stream writer
 ///
@@ -45,27 +31,45 @@ pub struct StreamWriter<W: Write> {
     finished: bool,
     /// Keeps track of dictionaries that have been written
     dictionary_tracker: DictionaryTracker,
+
+    ipc_fields: Option<Vec<IpcField>>,
 }
 
 impl<W: Write> StreamWriter<W> {
-    /// Try create a new writer, with the schema written as part of the header
-    pub fn try_new(mut writer: W, schema: &Schema, write_options: WriteOptions) -> Result<Self> {
-        // write the schema, set the written bytes to the schema
-        let encoded_message = EncodedData {
-            ipc_message: schema_to_bytes(schema),
-            arrow_data: vec![],
-        };
-        write_message(&mut writer, encoded_message)?;
-        Ok(Self {
+    /// Creates a new [`StreamWriter`]
+    pub fn new(writer: W, write_options: WriteOptions) -> Self {
+        Self {
             writer,
             write_options,
             finished: false,
             dictionary_tracker: DictionaryTracker::new(false),
-        })
+            ipc_fields: None,
+        }
     }
 
-    /// Write a record batch to the stream
-    pub fn write(&mut self, batch: &RecordBatch) -> Result<()> {
+    /// Starts the stream by writing a Schema message to it.
+    /// Use `ipc_fields` to declare dictionary ids in the schema, for dictionary-reuse
+    pub fn start(&mut self, schema: &Schema, ipc_fields: Option<Vec<IpcField>>) -> Result<()> {
+        self.ipc_fields = Some(if let Some(ipc_fields) = ipc_fields {
+            ipc_fields
+        } else {
+            default_ipc_fields(&schema.fields)
+        });
+
+        let encoded_message = EncodedData {
+            ipc_message: schema_to_bytes(schema, self.ipc_fields.as_ref().unwrap()),
+            arrow_data: vec![],
+        };
+        write_message(&mut self.writer, encoded_message)?;
+        Ok(())
+    }
+
+    /// Writes [`Chunk`] to the stream
+    pub fn write(
+        &mut self,
+        columns: &Chunk<Arc<dyn Array>>,
+        ipc_fields: Option<&[IpcField]>,
+    ) -> Result<()> {
         if self.finished {
             return Err(ArrowError::Io(std::io::Error::new(
                 std::io::ErrorKind::UnexpectedEof,
@@ -73,8 +77,16 @@ impl<W: Write> StreamWriter<W> {
             )));
         }
 
-        let (encoded_dictionaries, encoded_message) =
-            encoded_batch(batch, &mut self.dictionary_tracker, &self.write_options)?;
+        // we can't make it a closure because it borrows (and it can't borrow mut and non-mut below)
+        #[allow(clippy::or_fun_call)]
+        let fields = ipc_fields.unwrap_or(self.ipc_fields.as_ref().unwrap());
+
+        let (encoded_dictionaries, encoded_message) = encode_chunk(
+            columns,
+            fields,
+            &mut self.dictionary_tracker,
+            &self.write_options,
+        )?;
 
         for encoded_dictionary in encoded_dictionaries {
             write_message(&mut self.writer, encoded_dictionary)?;
