@@ -1,6 +1,10 @@
 #![forbid(unsafe_code)]
 //! Contains modules to interface with other formats such as [`csv`],
 //! [`parquet`], [`json`], [`ipc`], [`mod@print`] and [`avro`].
+
+#[cfg(feature = "io_odbc")]
+pub mod odbc;
+
 #[cfg(any(
     feature = "io_csv_read",
     feature = "io_csv_read_async",
@@ -19,6 +23,9 @@ pub mod csv;
 #[cfg(feature = "io_json")]
 #[cfg_attr(docsrs, doc(cfg(feature = "io_json")))]
 pub mod json;
+#[cfg(feature = "io_json")]
+#[cfg_attr(docsrs, doc(cfg(feature = "io_json")))]
+pub mod ndjson;
 
 #[cfg(feature = "io_ipc")]
 #[cfg_attr(docsrs, doc(cfg(feature = "io_ipc")))]
