@@ -1,25 +1,23 @@
-use parquet2::{
-    encoding::Encoding, metadata::ColumnDescriptor, page::DataPage, write::WriteOptions,
-};
+use parquet2::{encoding::Encoding, metadata::Descriptor, page::DataPage};
 
-use super::super::{levels, utils};
+use super::super::{levels, utils, WriteOptions};
 use super::basic::{build_statistics, encode_plain};
+use crate::io::parquet::read::schema::is_nullable;
 use crate::{
     array::{Array, BooleanArray, Offset},
     error::Result,
-    io::parquet::read::is_type_nullable,
 };
 
 pub fn array_to_page<O>(
     array: &BooleanArray,
     options: WriteOptions,
-    descriptor: ColumnDescriptor,
+    descriptor: Descriptor,
     nested: levels::NestedInfo<O>,
 ) -> Result<DataPage>
 where
     O: Offset,
 {
-    let is_optional = is_type_nullable(descriptor.type_());
+    let is_optional = is_nullable(&descriptor.primitive_type.field_info);
 
     let validity = array.validity();
 
@@ -41,6 +39,7 @@ where
     utils::build_plain_page(
         buffer,
         levels::num_values(nested.offsets()),
+        nested.offsets().len().saturating_sub(1),
         array.null_count(),
         repetition_levels_byte_length,
         definition_levels_byte_length,
