@@ -16,25 +16,35 @@ use crate::array::*;
 use crate::bitmap::Bitmap;
 use crate::datatypes::*;
 use crate::error::{ArrowError, Result};
-use crate::io::parquet::read::is_type_nullable;
+use crate::io::parquet::read::schema::is_nullable;
 use crate::io::parquet::write::levels::NestedInfo;
 use crate::types::days_ms;
 use crate::types::NativeType;
 
 use parquet2::page::DataPage;
 pub use parquet2::{
-    compression::Compression,
+    compression::CompressionOptions,
     encoding::Encoding,
     fallible_streaming_iterator,
-    metadata::{ColumnDescriptor, KeyValue, SchemaDescriptor},
+    metadata::{Descriptor, KeyValue, SchemaDescriptor},
     page::{CompressedDataPage, CompressedPage, EncodedPage},
     schema::types::ParquetType,
-    write::{
-        compress, Compressor, DynIter, DynStreamingIterator, RowGroupIter, Version, WriteOptions,
-    },
+    write::{compress, Compressor, DynIter, DynStreamingIterator, RowGroupIter, Version},
     FallibleStreamingIterator,
 };
 
+/// Currently supported options to write to parquet
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub struct WriteOptions {
+    /// Whether to write statistics
+    pub write_statistics: bool,
+    /// The page and file version to use
+    pub version: Version,
+    /// The compression to apply to every page
+    pub compression: CompressionOptions,
+}
+
+use crate::compute::aggregate::estimated_bytes_size;
 pub use file::FileWriter;
 pub use row_group::{row_group_iter, RowGroupIterator};
 pub use schema::to_parquet_type;
@@ -80,30 +90,43 @@ pub fn can_encode(data_type: &DataType, encoding: Encoding) -> bool {
 /// Returns an iterator of [`EncodedPage`].
 pub fn array_to_pages(
     array: &dyn Array,
-    descriptor: ColumnDescriptor,
+    descriptor: Descriptor,
     options: WriteOptions,
     encoding: Encoding,
 ) -> Result<DynIter<'static, Result<EncodedPage>>> {
-    match array.data_type() {
-        DataType::Dictionary(key_type, _, _) => {
-            match_integer_type!(key_type, |$T| {
-                dictionary::array_to_pages::<$T>(
-                    array.as_any().downcast_ref().unwrap(),
-                    descriptor,
-                    options,
-                    encoding,
-                )
-            })
+    // maximum page size is 2^31 e.g. i32::MAX
+    // we split at 2^30 to err on the safe side
+    if estimated_bytes_size(array) >= 2u32.pow(30) as usize {
+        let split_at = array.len() / 2;
+        let left = array.slice(0, split_at);
+        let right = array.slice(split_at, array.len() - split_at);
+
+        Ok(DynIter::new(
+            array_to_pages(&*left, descriptor.clone(), options, encoding)?
+                .chain(array_to_pages(&*right, descriptor, options, encoding)?),
+        ))
+    } else {
+        match array.data_type() {
+            DataType::Dictionary(key_type, _, _) => {
+                match_integer_type!(key_type, |$T| {
+                    dictionary::array_to_pages::<$T>(
+                        array.as_any().downcast_ref().unwrap(),
+                        descriptor,
+                        options,
+                        encoding,
+                    )
+                })
+            }
+            _ => array_to_page(array, descriptor, options, encoding)
+                .map(|page| DynIter::new(std::iter::once(Ok(page)))),
         }
-        _ => array_to_page(array, descriptor, options, encoding)
-            .map(|page| DynIter::new(std::iter::once(Ok(page)))),
     }
 }
 
 /// Converts an [`Array`] to a [`CompressedPage`] based on options, descriptor and `encoding`.
 pub fn array_to_page(
     array: &dyn Array,
-    descriptor: ColumnDescriptor,
+    descriptor: Descriptor,
     options: WriteOptions,
     encoding: Encoding,
 ) -> Result<EncodedPage> {
@@ -220,7 +243,15 @@ pub fn array_to_page(
                 values.into(),
                 array.validity().cloned(),
             );
-            fixed_len_bytes::array_to_page(&array, options, descriptor)
+            let statistics = if options.write_statistics {
+                Some(fixed_len_bytes::build_statistics(
+                    &array,
+                    descriptor.primitive_type.clone(),
+                ))
+            } else {
+                None
+            };
+            fixed_len_bytes::array_to_page(&array, options, descriptor, statistics)
         }
         DataType::Interval(IntervalUnit::DayTime) => {
             let array = array
@@ -238,13 +269,29 @@ pub fn array_to_page(
                 values.into(),
                 array.validity().cloned(),
             );
-            fixed_len_bytes::array_to_page(&array, options, descriptor)
+            let statistics = if options.write_statistics {
+                Some(fixed_len_bytes::build_statistics(
+                    &array,
+                    descriptor.primitive_type.clone(),
+                ))
+            } else {
+                None
+            };
+            fixed_len_bytes::array_to_page(&array, options, descriptor, statistics)
+        }
+        DataType::FixedSizeBinary(_) => {
+            let array = array.as_any().downcast_ref().unwrap();
+            let statistics = if options.write_statistics {
+                Some(fixed_len_bytes::build_statistics(
+                    array,
+                    descriptor.primitive_type.clone(),
+                ))
+            } else {
+                None
+            };
+
+            fixed_len_bytes::array_to_page(array, options, descriptor, statistics)
         }
-        DataType::FixedSizeBinary(_) => fixed_len_bytes::array_to_page(
-            array.as_any().downcast_ref().unwrap(),
-            options,
-            descriptor,
-        ),
         DataType::Decimal(precision, _) => {
             let precision = *precision;
             let array = array
@@ -275,6 +322,18 @@ pub fn array_to_page(
                 primitive::array_to_page::<i64, i64>(&array, options, descriptor)
             } else {
                 let size = decimal_length_from_precision(precision);
+
+                let statistics = if options.write_statistics {
+                    let stats = fixed_len_bytes::build_statistics_decimal(
+                        array,
+                        descriptor.primitive_type.clone(),
+                        size,
+                    );
+                    Some(stats)
+                } else {
+                    None
+                };
+
                 let mut values = Vec::<u8>::with_capacity(size * array.len());
                 array.values().iter().for_each(|x| {
                     let bytes = &x.to_be_bytes()[16 - size..];
@@ -285,7 +344,7 @@ pub fn array_to_page(
                     values.into(),
                     array.validity().cloned(),
                 );
-                fixed_len_bytes::array_to_page(&array, options, descriptor)
+                fixed_len_bytes::array_to_page(&array, options, descriptor, statistics)
             }
         }
         DataType::FixedSizeList(_, _) | DataType::List(_) | DataType::LargeList(_) => {
@@ -316,11 +375,11 @@ fn list_array_to_page<O: Offset>(
     offsets: &[O],
     validity: Option<&Bitmap>,
     values: &dyn Array,
-    descriptor: ColumnDescriptor,
+    descriptor: Descriptor,
     options: WriteOptions,
 ) -> Result<DataPage> {
     use DataType::*;
-    let is_optional = is_type_nullable(descriptor.type_());
+    let is_optional = is_nullable(&descriptor.primitive_type.field_info);
     let nested = NestedInfo::new(offsets, validity, is_optional);
 
     match values.data_type() {
@@ -347,47 +406,19 @@ fn list_array_to_page<O: Offset>(
 
         Utf8 => {
             let values = values.as_any().downcast_ref().unwrap();
-            let is_optional = is_type_nullable(descriptor.type_());
-
-            utf8::nested_array_to_page::<i32, O>(
-                values,
-                options,
-                descriptor,
-                NestedInfo::new(offsets, validity, is_optional),
-            )
+            utf8::nested_array_to_page::<i32, O>(values, options, descriptor, nested)
         }
         LargeUtf8 => {
             let values = values.as_any().downcast_ref().unwrap();
-            let is_optional = is_type_nullable(descriptor.type_());
-
-            utf8::nested_array_to_page::<i64, O>(
-                values,
-                options,
-                descriptor,
-                NestedInfo::new(offsets, validity, is_optional),
-            )
+            utf8::nested_array_to_page::<i64, O>(values, options, descriptor, nested)
         }
         Binary => {
             let values = values.as_any().downcast_ref().unwrap();
-            let is_optional = is_type_nullable(descriptor.type_());
-
-            binary::nested_array_to_page::<i32, O>(
-                values,
-                options,
-                descriptor,
-                NestedInfo::new(offsets, validity, is_optional),
-            )
+            binary::nested_array_to_page::<i32, O>(values, options, descriptor, nested)
         }
         LargeBinary => {
             let values = values.as_any().downcast_ref().unwrap();
-            let is_optional = is_type_nullable(descriptor.type_());
-
-            binary::nested_array_to_page::<i64, O>(
-                values,
-                options,
-                descriptor,
-                NestedInfo::new(offsets, validity, is_optional),
-            )
+            binary::nested_array_to_page::<i64, O>(values, options, descriptor, nested)
         }
         _ => todo!(),
     }
@@ -395,7 +426,7 @@ fn list_array_to_page<O: Offset>(
 
 fn nested_array_to_page(
     array: &dyn Array,
-    descriptor: ColumnDescriptor,
+    descriptor: Descriptor,
     options: WriteOptions,
 ) -> Result<DataPage> {
     match array.data_type() {
@@ -421,7 +452,7 @@ fn nested_array_to_page(
         }
         DataType::FixedSizeList(_, size) => {
             let array = array.as_any().downcast_ref::<FixedSizeListArray>().unwrap();
-            let offsets = (0..array.len())
+            let offsets = (0..=array.len())
                 .map(|x| (*size * x) as i32)
                 .collect::<Vec<_>>();
             list_array_to_page(
