@@ -2,10 +2,12 @@ use crate::datatypes::DataType;
 
 use super::Scalar;
 
-#[derive(Debug, Clone, PartialEq)]
+/// The representation of a single entry of a [`crate::array::NullArray`].
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub struct NullScalar {}
 
 impl NullScalar {
+    /// A new [`NullScalar`]
     #[inline]
     pub fn new() -> Self {
         Self {}
