@@ -27,34 +27,23 @@ fn read() -> Result<()> {
 "Aberdeen, Aberdeen City, UK",57.149651,-2.099075"#;
     let mut reader = ReaderBuilder::new().from_reader(Cursor::new(data));
 
-    let schema = Arc::new(infer_schema(&mut reader, None, true, &infer)?);
+    let (fields, _) = infer_schema(&mut reader, None, true, &infer)?;
 
     let mut rows = vec![ByteRecord::default(); 100];
     let rows_read = read_rows(&mut reader, 0, &mut rows)?;
 
-    let batch = deserialize_batch(
-        &rows[..rows_read],
-        schema.fields(),
-        None,
-        0,
-        deserialize_column,
-    )?;
+    let columns = deserialize_batch(&rows[..rows_read], &fields, None, 0, deserialize_column)?;
 
-    let batch_schema = batch.schema();
+    assert_eq!(14, columns.len());
+    assert_eq!(3, columns.arrays().len());
 
-    assert_eq!(&schema, batch_schema);
-    assert_eq!(14, batch.num_rows());
-    assert_eq!(3, batch.num_columns());
-
-    let lat = batch
-        .column(1)
+    let lat = columns.arrays()[1]
         .as_any()
         .downcast_ref::<Float64Array>()
         .unwrap();
     assert!((57.653484 - lat.value(0)).abs() < f64::EPSILON);
 
-    let city = batch
-        .column(0)
+    let city = columns.arrays()[0]
         .as_any()
         .downcast_ref::<Utf8Array<i32>>()
         .unwrap();
@@ -69,15 +58,15 @@ fn infer_basics() -> Result<()> {
     let file = Cursor::new("1,2,3\na,b,c\na,,c");
     let mut reader = ReaderBuilder::new().from_reader(file);
 
-    let schema = infer_schema(&mut reader, Some(10), false, &infer)?;
+    let (fields, _) = infer_schema(&mut reader, Some(10), false, &infer)?;
 
     assert_eq!(
-        schema,
-        Schema::new(vec![
+        fields,
+        vec![
             Field::new("column_1", DataType::Utf8, true),
             Field::new("column_2", DataType::Utf8, true),
             Field::new("column_3", DataType::Utf8, true),
-        ])
+        ]
     );
     Ok(())
 }
@@ -87,15 +76,15 @@ fn infer_ints() -> Result<()> {
     let file = Cursor::new("1,2,3\n1,a,5\n2,,4");
     let mut reader = ReaderBuilder::new().from_reader(file);
 
-    let schema = infer_schema(&mut reader, Some(10), false, &infer)?;
+    let (fields, _) = infer_schema(&mut reader, Some(10), false, &infer)?;
 
     assert_eq!(
-        schema,
-        Schema::new(vec![
+        fields,
+        vec![
             Field::new("column_1", DataType::Int64, true),
             Field::new("column_2", DataType::Utf8, true),
             Field::new("column_3", DataType::Int64, true),
-        ])
+        ]
     );
     Ok(())
 }
