@@ -1,5 +1,7 @@
 mod binary;
 mod boolean;
+mod fixed_size_binary;
+mod fixed_size_list;
 mod list;
 mod null;
 mod primitive;
