@@ -5,7 +5,7 @@ use arrow2::types::months_days_ns;
 
 #[test]
 fn naive() {
-    let expected = "Timestamp(Nanosecond, None)[1996-12-19 16:39:57, 1996-12-19 13:39:57, ]";
+    let expected = "Timestamp(Nanosecond, None)[1996-12-19 16:39:57, 1996-12-19 13:39:57, None]";
     let fmt = "%Y-%m-%dT%H:%M:%S:z";
     let array = Utf8Array::<i32>::from_slice(&[
         "1996-12-19T16:39:57-02:00",
@@ -13,7 +13,7 @@ fn naive() {
         "1996-12-19 13:39:57-03:00", // missing T
     ]);
     let r = temporal_conversions::utf8_to_naive_timestamp_ns(&array, fmt);
-    assert_eq!(format!("{}", r), expected);
+    assert_eq!(format!("{:?}", r), expected);
 
     let fmt = "%Y-%m-%dT%H:%M:%S"; // no tz info
     let array = Utf8Array::<i32>::from_slice(&[
@@ -22,12 +22,12 @@ fn naive() {
         "1996-12-19 13:39:57-03:00", // missing T
     ]);
     let r = temporal_conversions::utf8_to_naive_timestamp_ns(&array, fmt);
-    assert_eq!(format!("{}", r), expected);
+    assert_eq!(format!("{:?}", r), expected);
 }
 
 #[test]
 fn naive_no_tz() {
-    let expected = "Timestamp(Nanosecond, None)[1996-12-19 16:39:57, 1996-12-19 13:39:57, ]";
+    let expected = "Timestamp(Nanosecond, None)[1996-12-19 16:39:57, 1996-12-19 13:39:57, None]";
     let fmt = "%Y-%m-%dT%H:%M:%S"; // no tz info
     let array = Utf8Array::<i32>::from_slice(&[
         "1996-12-19T16:39:57",
@@ -35,14 +35,14 @@ fn naive_no_tz() {
         "1996-12-19 13:39:57", // missing T
     ]);
     let r = temporal_conversions::utf8_to_naive_timestamp_ns(&array, fmt);
-    assert_eq!(format!("{}", r), expected);
+    assert_eq!(format!("{:?}", r), expected);
 }
 
 #[test]
 fn tz_aware() {
     let tz = "-02:00".to_string();
     let expected =
-        "Timestamp(Nanosecond, Some(\"-02:00\"))[1996-12-19 16:39:57 -02:00, 1996-12-19 17:39:57 -02:00, ]";
+        "Timestamp(Nanosecond, Some(\"-02:00\"))[1996-12-19 16:39:57 -02:00, 1996-12-19 17:39:57 -02:00, None]";
     let fmt = "%Y-%m-%dT%H:%M:%S%.f%:z";
     let array = Utf8Array::<i32>::from_slice(&[
         "1996-12-19T16:39:57.0-02:00",
@@ -50,13 +50,13 @@ fn tz_aware() {
         "1996-12-19 13:39:57.0-03:00",
     ]);
     let r = temporal_conversions::utf8_to_timestamp_ns(&array, fmt, tz).unwrap();
-    assert_eq!(format!("{}", r), expected);
+    assert_eq!(format!("{:?}", r), expected);
 }
 
 #[test]
 fn tz_aware_no_timezone() {
     let tz = "-02:00".to_string();
-    let expected = "Timestamp(Nanosecond, Some(\"-02:00\"))[, , ]";
+    let expected = "Timestamp(Nanosecond, Some(\"-02:00\"))[None, None, None]";
     let fmt = "%Y-%m-%dT%H:%M:%S%.f";
     let array = Utf8Array::<i32>::from_slice(&[
         "1996-12-19T16:39:57.0",
@@ -64,7 +64,7 @@ fn tz_aware_no_timezone() {
         "1996-12-19 13:39:57.0",
     ]);
     let r = temporal_conversions::utf8_to_timestamp_ns(&array, fmt, tz).unwrap();
-    assert_eq!(format!("{}", r), expected);
+    assert_eq!(format!("{:?}", r), expected);
 }
 
 #[test]
